import calendar
import datetime
import time

import six
import tzlocal


def _encode_datetime(dttime):
    if (hasattr(dttime, 'tzinfo') and dttime.tzinfo and
            dttime.tzinfo.utcoffset(dttime) is not None):
        utc_timestamp = calendar.timegm(dttime.utctimetuple())
    else:
        if isinstance(dttime, datetime.date):
            dttime = naive_date_to_datetime(dttime)
        local_datetime = tzlocal.get_localzone().localize(dttime)
        utc_timestamp = aware_datetime_to_epoch_seconds(local_datetime)

    return int(utc_timestamp)


def encode(data):
    for key, value in six.iteritems(data):
        if value is None:
            continue
        elif isinstance(value, list) or isinstance(value, tuple):
            for subvalue in value:
                yield ("%s[]" % (key,), subvalue)
        elif isinstance(value, dict):
            subdict = dict(('%s[%s]' % (key, subkey), subvalue) for
                           subkey, subvalue in six.iteritems(value))
            for subkey, subvalue in encode(subdict):
                yield (subkey, subvalue)
        elif (isinstance(value, datetime.datetime) or
              isinstance(value, datetime.date)):
            yield (key, _encode_datetime(value))
        else:
            yield (key, value)


def naive_date_to_datetime(naive_date):
    return datetime.datetime(*naive_date.timetuple()[:6])


def aware_datetime_to_epoch_seconds(localized_datetime):
    raw_seconds = time.mktime(localized_datetime.utctimetuple())
    epoch_seconds = time.mktime((1970, 1, 1, 0, 0, 0, 0, 0, 0))
    return raw_seconds - epoch_seconds



__title__ = 'delighted'
__version__ = '3.0.0rc1'
__author__ = 'Ben Turner'
__license__ = 'MIT'

from delighted.http_adapter import HTTPAdapter  # noqa

api_key = None
api_base_url = 'https://api.delightedapp.com/v1/'
http_adapter = HTTPAdapter()
shared_client = None


def get_shared_client():
    global shared_client
    if not shared_client:
        shared_client = Client()
    return shared_client


from delighted.client import Client  # noqa
from delighted.resource import (  # noqa
    Metrics,
    Person,
    SurveyRequest,
    SurveyResponse,
    Unsubscribe,
    Bounce,
)



from base64 import b64encode
import json
from six import b
from six.moves.urllib_parse import urljoin

import delighted
from delighted.errors import (
    AuthenticationError,
    GeneralAPIError,
    ResourceValidationError,
    ServiceUnavailableError,
    TooManyRequestsError,
    UnsupportedFormatRequestedError,
)
from delighted.util import encode


class Client(object):

    def __init__(self, api_key=None, api_base_url=None, http_adapter=None):
        self.api_key = api_key or delighted.api_key
        self.api_base_url = api_base_url or delighted.api_base_url
        self.http_adapter = http_adapter or delighted.http_adapter

        if self.api_key is None:
            raise ValueError("You must provide an API key by setting " +
                             "delighted.api_key = '123abc' or passing " +
                             "api_key='abc123' when instantiating client.")

    def request(self, method, resource, headers={}, params={}):
        headers['Accept'] = 'application/json'
        headers['Authorization'] = 'Basic %s' % \
            (b64encode(b(self.api_key)).decode('ascii'))
        headers['User-Agent'] = "Delighted Python %s" % delighted.__version__
        if method in ('post', 'put', 'delete'):
            headers['Content-Type'] = 'application/json'

        url = urljoin(self.api_base_url, resource)

        if method == 'get' and params:
            params = dict((key, value) for (key, value) in encode(params))
            data = None
        else:
            data = json.dumps(params)
            params = None

        response = self.http_adapter.request(method, url, headers, data,
                                             params)
        return self._handle_response(response)

    def _handle_response(self, response):
        if response.status_code in (200, 201, 202):
            return json.loads(response.body)
        if response.status_code == 401:
            raise AuthenticationError(response)
        if response.status_code == 406:
            raise UnsupportedFormatRequestedError(response)
        if response.status_code == 422:
            raise ResourceValidationError(response)
        if response.status_code == 429:
            raise TooManyRequestsError(response)
        if response.status_code == 503:
            raise ServiceUnavailableError(response)
        raise GeneralAPIError(response)



import requests

from delighted.http_response import HTTPResponse


class HTTPAdapter(object):

    def request(self, method, uri, headers={}, data=None, params=None):
        resp = requests.request(method, uri, headers=headers, data=data,
                                params=params)

        return HTTPResponse(resp.status_code, resp.headers, resp.text)



import unittest

import delighted


class ClientTest(unittest.TestCase):

    def test_instantiating_client_requires_api_key(self):
        original_api_key = delighted.api_key
        try:
            delighted.api_key = None
            self.assertRaises(ValueError, lambda: delighted.Client())
            delighted.Client(api_key='abc123')
        except:
            delighted.api_key = original_api_key



import re

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

version = ''
with open('delighted/__init__.py', 'r') as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', fd.read(),
                        re.MULTILINE).group(1)
if not version:
    raise RuntimeError('Cannot find version information')

setup(
    name='delighted',
    version=version,
    description='Delighted API Python Client.',
    long_description_content_type='text/markdown',
    long_description=open('README.md').read(),
    author='Ben Turner, Robby Colvin',
    author_email='ben@delighted.com',
    url='https://delighted.com/',
    packages=['delighted'],
    package_dir={'delighted': 'delighted'},
    install_requires=['six', 'requests', 'tzlocal'],
    test_suite='test',
    test_requires=['mock', 'pytz', 'tzlocal'],
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
)



import delighted
from delighted.util import aware_datetime_to_epoch_seconds, \
    naive_date_to_datetime
from . import get_headers, post_headers, DelightedTestCase
import datetime
import pytz
import tzlocal
from base64 import b64encode
from six import b
import sys

class TestResource(DelightedTestCase):
    def setUp(self):
        super(TestResource, self).setUp()

    def check_retrieving_metrics(self, client=None):
        data = {'nps': 10}
        url = 'https://api.delightedapp.com/v1/metrics'
        self.mock_response(200, {}, data)

        expected_headers = get_headers.copy()
        retrieve_kwargs = {}
        if client:
            retrieve_kwargs['client'] = client
            expected_headers['Authorization'] = 'Basic %s' % b64encode(b(client.api_key)).decode('ascii')

        metrics = delighted.Metrics.retrieve(**retrieve_kwargs)
        self.check_call('get', url, expected_headers, {}, None)
        self.assertTrue(delighted.Metrics is type(metrics))
        self.assertEqual(dict(metrics), data)
        self.assertEqual(metrics.nps, 10)
        self.assertRaises(AttributeError, lambda: metrics.id)

    def test_retrieving_metrics(self):
        self.check_retrieving_metrics()

    def test_retrieving_metrics_other_client(self):
        client = delighted.Client(api_key='example')
        self.check_retrieving_metrics(client=client)

    def test_retrieving_metrics_range_unixtimestamp(self):
        data = {'nps': 10}
        self.mock_response(200, {}, data)
        since = 1425168000
        until = 1430348400
        url = 'https://api.delightedapp.com/v1/metrics'

        metrics = delighted.Metrics.retrieve(since=since, until=until)
        self.check_call('get', url, get_headers, None,
                        {'since': 1425168000, 'until': 1430348400})
        self.assertTrue(delighted.Metrics is type(metrics))
        self.assertEqual(dict(metrics), data)
        self.assertEqual(metrics.nps, 10)
        self.assertRaises(AttributeError, lambda: metrics.id)

    def test_retrieving_metrics_range_date_object(self):
        data = {'nps': 10}
        self.mock_response(200, {}, data)
        since = datetime.date(2013, 10, 1)
        until = datetime.date(2013, 11, 1)
        timezone = tzlocal.get_localzone()
        since_seconds = self._naive_date_to_epoch_seconds(since, timezone)
        until_seconds = self._naive_date_to_epoch_seconds(until, timezone)
        url = 'https://api.delightedapp.com/v1/metrics'

        metrics = delighted.Metrics.retrieve(since=since, until=until)
        self.check_call('get', url, get_headers, None,
                        {'since': since_seconds, 'until': until_seconds})
        self.assertTrue(delighted.Metrics is type(metrics))
        self.assertEqual(dict(metrics), data)
        self.assertEqual(metrics.nps, 10)
        self.assertRaises(AttributeError, lambda: metrics.id)

    def test_retrieving_metrics_range_datetime_object(self):
        data = {'nps': 10}
        self.mock_response(200, {}, data)
        timezone = pytz.timezone('America/Chicago')
        since = timezone.localize(datetime.datetime(2013, 10, 1))
        until = timezone.localize(datetime.datetime(2013, 11, 1))
        url = 'https://api.delightedapp.com/v1/metrics'

        metrics = delighted.Metrics.retrieve(since=since, until=until)
        self.check_call('get', url, get_headers, None,
                        {'since': 1380603600, 'until': 1383282000})
        self.assertTrue(delighted.Metrics is type(metrics))
        self.assertEqual(dict(metrics), data)
        self.assertEqual(metrics.nps, 10)
        self.assertRaises(AttributeError, lambda: metrics.id)

    def check_creating_or_updating_a_person(self, client=None):
        email = 'foo@bar.com'
        data = {'id': '123', 'email': email}
        url = 'https://api.delightedapp.com/v1/people'
        self.mock_response(200, {}, data)

        expected_headers = post_headers.copy()
        create_kwargs = {'email': email}
        if client:
            create_kwargs['client'] = client
            expected_headers['Authorization'] = 'Basic %s' % b64encode(b(client.api_key)).decode('ascii')

        person = delighted.Person.create(**create_kwargs)
        self.assertTrue(delighted.Person is type(person))
        self.assertEqual(person, {'id': '123', 'email': email})
        self.assertEqual(person.email, email)
        self.assertEqual('123', person.id)
        self.check_call('post', url, expected_headers, {'email': email}, None)

    def test_creating_or_updating_a_person(self):
        self.check_creating_or_updating_a_person()

    def test_creating_or_updating_a_person_other_client(self):
        client = delighted.Client(api_key='example')
        self.check_creating_or_updating_a_person(client=client)

    def test_unsubscribing_a_person(self):
        email = 'person@example.com'
        data = {'person_email': email}
        url = 'https://api.delightedapp.com/v1/unsubscribes'
        self.mock_response(200, {}, {'ok': True})

        delighted.Unsubscribe.create(person_email=email)
        self.check_call('post', url, post_headers, data, None)

    def test_deleting_a_person_by_multiple_identifiers(self):
        self.assertRaises(ValueError, lambda: delighted.Person.delete(id=42, email="foo@example.com"))

    def test_deleting_a_person_by_id(self):
        url = 'https://api.delightedapp.com/v1/people/42'
        self.mock_response(202, {}, {'ok': True})

        delighted.Person.delete(id=42)
        self.check_call('delete', url, post_headers, {}, None)

    def test_deleting_a_person_by_email(self):
        url = 'https://api.delightedapp.com/v1/people/email%3Afoo%40example.com'
        self.mock_response(202, {}, {'ok': True})

        delighted.Person.delete(email='foo@example.com')
        self.check_call('delete', url, post_headers, {}, None)

    def test_deleting_a_person_by_phone_number(self):
        url = 'https://api.delightedapp.com/v1/people/phone_number%3A%2B14155551212'
        self.mock_response(202, {}, {'ok': True})

        delighted.Person.delete(phone_number='+14155551212')
        self.check_call('delete', url, post_headers, {}, None)


    def test_deleting_pending_survey_requests_for_a_person(self):
        email = 'foo@bar.com'
        url = 'https://api.delightedapp.com/v1/people/foo%40bar.com' + \
              '/survey_requests/pending'
        self.mock_response(200, {}, {'ok': True})

        result = delighted.SurveyRequest.delete_pending(person_email=email)
        self.assertTrue(dict is type(result))
        self.assertEqual({'ok': True}, result)
        self.check_call('delete', url, post_headers, {}, None)

    def test_creating_a_survey_response(self):
        url = 'https://api.delightedapp.com/v1/survey_responses'
        data = {'id': '456', 'person': '123', 'score': 10}
        self.mock_response(200, {}, data)
        survey_response = delighted.SurveyResponse.create(person='123',
                                                          score=10)
        self.assertTrue(delighted.SurveyResponse is type(survey_response))
        self.assertEqual({'id': '456', 'person': '123', 'score': 10}, survey_response)
        self.assertEqual('123', survey_response.person)
        self.assertEqual(10, survey_response.score)
        self.assertEqual('456', survey_response.id)
        resp = {'person': '123', 'score': 10}
        self.check_call('post', url, post_headers, resp, None)

    def test_retrieving_a_survey_response_expand_person(self):
        url = 'https://api.delightedapp.com/v1/survey_responses/456'
        resp = {'id': '456',
                'person': {'id': '123', 'email': 'foo@bar.com', 'type': 'aaa'},
                'score': 10}
        self.mock_response(200, {}, resp)

        survey_response = delighted.SurveyResponse.retrieve('456',
                                                            expand=['person'])
        self.check_call('get', url, get_headers, None, {'expand[]': 'person'})
        self.assertTrue(delighted.SurveyResponse is type(survey_response))
        self.assertTrue(delighted.Person is type(survey_response.person))
        self.assertEqual(resp, survey_response)
        self.assertEqual(
            {'id': '123', 'email': 'foo@bar.com', 'type': 'aaa'},
            survey_response.person
        )
        self.assertEqual('123', survey_response.person.id)
        self.assertEqual('foo@bar.com', survey_response.person.email)
        self.assertEqual('aaa', survey_response.person.type)
        self.assertEqual(10, survey_response.score)
        self.assertEqual('456', survey_response.id)

    def test_updating_a_survey_response(self):
        url = 'https://api.delightedapp.com/v1/survey_responses/456'
        data = {'person': '123', 'score': 10}
        resp = {'id': '456', 'person': '123', 'score': 10}
        self.mock_response(200, {}, resp)

        old = {'id': '456', 'person': '321', 'score': 1}
        survey_response = delighted.SurveyResponse(old)
        survey_response.person = '123'
        survey_response.score = 10
        self.assertTrue(delighted.SurveyResponse is type(survey_response.save()))
        self.check_call('put', url, post_headers, resp, None)
        self.assertEqual(resp, survey_response)
        self.assertEqual('123', survey_response.person)
        self.assertEqual(10, survey_response.score)
        self.assertEqual('456', survey_response.id)

    def test_listing_all_survey_responses(self):
        url = 'https://api.delightedapp.com/v1/survey_responses'
        resp1 = {'id': '123', 'comment': 'One'}
        resp2 = {'id': '456', 'comment': 'Two'}
        self.mock_response(200, {}, [resp1, resp2])

        survey_responses = delighted.SurveyResponse.all(order='desc')
        self.check_call('get', url, get_headers, None, {'order': 'desc'})
        self.assertTrue(list is type(survey_responses))
        self.assertTrue(delighted.SurveyResponse is type(survey_responses[0]))
        self.assertEqual({'id': '123', 'comment': 'One'}, survey_responses[0])
        self.assertEqual('One', survey_responses[0].comment)
        self.assertEqual('123', survey_responses[0].id)
        self.assertTrue(delighted.SurveyResponse is type(survey_responses[1]))
        self.assertEqual({'id': '456', 'comment': 'Two'}, survey_responses[1])
        self.assertEqual('Two', survey_responses[1].comment)
        self.assertEqual('456', survey_responses[1].id)

    def test_listing_all_survey_responses_expand_person(self):
        url = 'https://api.delightedapp.com/v1/survey_responses'
        resp1 = {'id': '123', 'comment': 'One',
                 'person': {'id': '456', 'email': 'foo@bar.com', 'type': 'aaa'}}
        resp2 = {'id': '789', 'comment': 'Two',
                 'person': {'id': '012', 'email': 'koo@bar.com', 'type': 'bbb'}}
        self.mock_response(200, {}, [resp1, resp2])

        survey_responses = delighted.SurveyResponse.all(expand=['person'])
        self.check_call('get', url, get_headers, None, {'expand[]': 'person'})
        self.assertTrue(list is type(survey_responses))

        self.assertTrue(delighted.SurveyResponse is type(survey_responses[0]))
        self.assertEqual(resp1, survey_responses[0])
        self.assertEqual('One', survey_responses[0].comment)
        self.assertEqual('123', survey_responses[0].id)
        self.assertEqual(delighted.Person, type(survey_responses[0].person))
        self.assertEqual(
            {'id': '456', 'email': 'foo@bar.com', 'type': 'aaa'},
            survey_responses[0].person
        )
        self.assertEqual('456', survey_responses[0].person.id)
        self.assertEqual('foo@bar.com', survey_responses[0].person.email)
        self.assertEqual('aaa', survey_responses[0].person.type)

        self.assertTrue(delighted.SurveyResponse is type(survey_responses[1]))
        self.assertEqual(resp2, survey_responses[1])
        self.assertEqual('Two', survey_responses[1].comment)
        self.assertEqual('789', survey_responses[1].id)
        self.assertEqual(delighted.Person, type(survey_responses[1].person))
        self.assertEqual(
            {'id': '012', 'email': 'koo@bar.com', 'type': 'bbb'},
            survey_responses[1].person
        )
        self.assertEqual('012', survey_responses[1].person.id)
        self.assertEqual('koo@bar.com', survey_responses[1].person.email)
        self.assertEqual('bbb', survey_responses[1].person.type)

    def test_listing_all_people(self):
        url = 'https://api.delightedapp.com/v1/people'
        person1 = {'id': '123', 'email': 'foo@example.com', 'name': 'Foo Smith'}
        person2 = {'id': '456', 'email': 'bar@example.com', 'name': 'Bar Kim'}
        self.mock_response(200, {}, [person1, person2])

        people = delighted.Person.all()
        self.check_call('get', url, get_headers, {}, None)
        self.assertTrue(list is type(people))
        self.assertTrue(delighted.Person is type(people[0]))
        self.assertEqual({'id': '123', 'email': 'foo@example.com', 'name': 'Foo Smith'}, people[0])
        self.assertEqual('foo@example.com', people[0].email)
        self.assertEqual('Foo Smith', people[0].name)
        self.assertEqual('123', people[0].id)
        self.assertTrue(delighted.Person is type(people[1]))
        self.assertEqual({'id': '456', 'email': 'bar@example.com', 'name': 'Bar Kim'}, people[1])
        self.assertEqual('Bar Kim', people[1].name)
        self.assertEqual('bar@example.com', people[1].email)
        self.assertEqual('456', people[1].id)

    def test_listing_all_unsubscribes(self):
        url = 'https://api.delightedapp.com/v1/unsubscribes'
        resp1 = {'person_id': '123', 'email': 'foo@bar.com', 'name': 'Foo', 'unsubscribed_at': 1440621400}
        self.mock_response(200, {}, [resp1])

        unsubscribes = delighted.Unsubscribe.all()
        self.check_call('get', url, get_headers, {}, None)
        self.assertTrue(list is type(unsubscribes))
        self.assertTrue(delighted.Unsubscribe is type(unsubscribes[0]))
        self.assertEqual(resp1, dict(unsubscribes[0]))

    def test_listing_all_bounces(self):
        url = 'https://api.delightedapp.com/v1/bounces'
        resp1 = {'person_id': '123', 'email': 'foo@bar.com', 'name': 'Foo', 'bounced_at': 1440621400}
        self.mock_response(200, {}, [resp1])

        bounces = delighted.Bounce.all()
        self.check_call('get', url, get_headers, {}, None)
        self.assertTrue(list is type(bounces))
        self.assertTrue(delighted.Bounce is type(bounces[0]))
        self.assertEqual(resp1, dict(bounces[0]))

    def test_rate_limit_response(self, client=None):
        self.mock_response(429, {'Retry-After': '5'}, {})

        # https://docs.python.org/2/library/unittest.html#unittest.TestCase.assertRaises
        # Ability to use assertRaises() as a context manager was added in 2.7
        if sys.version_info < (2, 7):
            self.assertRaises(delighted.errors.TooManyRequestsError, lambda: delighted.Metrics.retrieve(client=client))
        else:
            with self.assertRaises(delighted.errors.TooManyRequestsError) as context:
                delighted.Metrics.retrieve(client=client)

            self.assertEqual(5, context.exception.retry_after)

    @classmethod
    def _naive_date_to_epoch_seconds(cls, date_obj, timezone):
        datetime_obj = timezone.localize(naive_date_to_datetime(date_obj))
        return aware_datetime_to_epoch_seconds(datetime_obj)



import six
from six.moves.urllib_parse import quote

from delighted import get_shared_client


class Resource(dict):

    def __init__(self, attrs={}):
        super(Resource, self).__init__()

        if 'id' in attrs:
            object.__setattr__(self, 'id', attrs['id'])

        if hasattr(self.__class__, 'expandable_attributes'):
            for attr, klass in six.iteritems(self.expandable_attributes):
                if attr in attrs and isinstance(attrs[attr], dict):
                    expandable_attrs = attrs.pop(attr)
                    item_id = expandable_attrs['id']
                    super(Resource, self).__setitem__(attr, klass(expandable_attrs))

        for k, v in six.iteritems(attrs):
            super(Resource, self).__setitem__(k, v)

    def __setattr__(self, k, v):
        if k[0] == '_' or k in self.__dict__:
            return super(Resource, self).__setattr__(k, v)
        else:
            self[k] = v

    def __getattr__(self, k):
        if k[0] == '_' and k is not '_attrs':
            raise AttributeError(k)

        try:
            return self[k]
        except KeyError as err:
            raise AttributeError(*err.args)

    @classmethod
    def _set_client(self, params):
        self.client = params.pop('client', None) or get_shared_client()

    @classmethod
    def _identifier_string(self, **identifier_dict):
        if len(identifier_dict) != 1:
            raise ValueError('You must pass exactly one identifier name and value')

        identifier_key = list(identifier_dict.keys())[0]
        identifier_value = list(identifier_dict.values())[0]

        if 'id' == str(identifier_key):
            return str(identifier_value)
        else:
            return '%s:%s' % (identifier_key, identifier_value)


class AllResource(Resource):
    @classmethod
    def all(self, **params):
        self._set_client(params)
        j = self.client.request('get', self.path, {}, params)
        return [self(i) for i in j]


class CreateableResource(Resource):

    @classmethod
    def create(self, **params):
        self._set_client(params)
        j = self.client.request('post', self.path, {}, params)
        return self(j)


class RetrievableResource(Resource):

    @classmethod
    def retrieve(self, *args, **params):
        self._set_client(params)
        path = '%s/%s' % (self.path, args[0]) if len(args) > 0 else self.path
        j = self.client.request('get', path, {}, params)
        return self(j)


class UpdateableResource(Resource):
    def save(self, **params):
        self._set_client(params)
        if hasattr(self.__class__, 'expandable_attributes'):
            expand_attrs = list(self.__class__.expandable_attributes.keys())
            params.update({'expand': expand_attrs})
        path = '%s/%s' % (self.path, self.id) if self.id else self.path
        j = self.client.request('put', path, {}, dict(self))
        return type(self)(j)


class DeleteableResource(Resource):
    @classmethod
    def delete(self, **params):
        self._set_client(params)
        identifier = self._identifier_string(**params)
        path = '%s/%s' % (self.path, quote(identifier, ''))
        return self.client.request('delete', path, {}, {})


class Metrics(RetrievableResource):
    path = 'metrics'
    singleton_resource = True


class Person(AllResource, CreateableResource, DeleteableResource):
    path = 'people'


class SurveyRequest(Resource):
    path = 'people/%s/survey_requests/pending'

    @classmethod
    def delete_pending(self, **params):
        if 'person_email' not in params:
            raise ValueError("You must pass person_email")

        self._set_client(params)

        escaped_email = quote(params['person_email'])
        url = self.path % escaped_email

        return self.client.request('delete', url)


class SurveyResponse(AllResource, CreateableResource,
                     RetrievableResource, UpdateableResource):
    expandable_attributes = {'person': Person}
    path = 'survey_responses'


class Unsubscribe(AllResource, CreateableResource):
    path = 'unsubscribes'


class Bounce(AllResource):
    path = 'bounces'



class DelightedError(Exception):

    def __init__(self, response):
        self.response = response

    def __repr__(self):
        return "<%d: %s>" % (self.response.status_code, self.response.body)

    __str__ = __repr__


class AuthenticationError(DelightedError):
    """401, api auth missing or incorrect."""
    pass


class UnsupportedFormatRequestedError(DelightedError):
    """406, invalid format in Accept header."""
    pass


class ResourceValidationError(DelightedError):
    """422, validation errors."""
    pass



class TooManyRequestsError(DelightedError):
    """429, rate limited."""

    def __init__(self, response):
        super(TooManyRequestsError, self).__init__(response)
        self.retry_after = int(response.headers['Retry-After'])



class GeneralAPIError(DelightedError):
    """500, general/unknown error."""
    pass


class ServiceUnavailableError(DelightedError):
    """503, maintenance or overloaded."""
    pass
